@@ -20,6 +20,8 @@
  */
 
 #include <stdio.h>
+#include <fcntl.h>
+#include <sys/stat.h>
 #include <windows.h>
 #include "fci.h"
 #include "fdi.h"
@@ -31,16 +33,6 @@
 
 static CHAR CURR_DIR[MAX_PATH];
 
-/* avoid including CRT headers */
-#ifndef _O_BINARY
-# define _O_BINARY 0x8000
-#endif
-
-#ifndef _S_IREAD
-# define _S_IREAD  0x0100
-# define _S_IWRITE 0x0080
-#endif
-
 #include <pshpack1.h>
 
 struct CFHEADER
@@ -744,7 +736,7 @@ static void test_FDIIsCabinet(void)
 
 static INT_PTR __cdecl CopyProgress(FDINOTIFICATIONTYPE fdint, PFDINOTIFICATION pfdin)
 {
-    return 0;
+    return 37;  /* doc says 0, but anything != -1 apparently means success as well */
 }
 
 static INT_PTR CDECL fdi_mem_open(char *name, int oflag, int pmode)
