@@ -39,6 +39,8 @@
 #include "device_private.h"
 #include "dinput_private.h"
 
+#define WM_WINE_NOTIFY_ACTIVITY WM_USER
+
 WINE_DEFAULT_DEBUG_CHANNEL(dinput);
 
 static inline IDirectInputDeviceImpl *impl_from_IDirectInputDevice8A(IDirectInputDevice8A *iface)
@@ -78,7 +80,7 @@ static void _dump_cooperativelevel_DI(DWORD dwFlags) {
 #undef FE
 	};
 	TRACE(" cooperative level : ");
-	for (i = 0; i < (sizeof(flags) / sizeof(flags[0])); i++)
+	for (i = 0; i < ARRAY_SIZE(flags); i++)
 	    if (flags[i].mask & dwFlags)
 		TRACE("%s ",flags[i].name);
 	TRACE("\n");
@@ -104,7 +106,7 @@ static void _dump_ObjectDataFormat_flags(DWORD dwFlags) {
     TRACE("Flags:");
 
     /* First the flags */
-    for (i = 0; i < (sizeof(flags) / sizeof(flags[0])); i++) {
+    for (i = 0; i < ARRAY_SIZE(flags); i++) {
         if (flags[i].mask & dwFlags)
         TRACE(" %s",flags[i].name);
     }
@@ -151,7 +153,7 @@ static void _dump_EnumObjects_flags(DWORD dwFlags) {
 	if (type == DIDFT_ALL) {
 	    TRACE(" DIDFT_ALL");
 	} else {
-	    for (i = 0; i < (sizeof(flags) / sizeof(flags[0])); i++) {
+	    for (i = 0; i < ARRAY_SIZE(flags); i++) {
 		if (flags[i].mask & type) {
 		    type &= ~flags[i].mask;
 		    TRACE(" %s",flags[i].name);
@@ -228,7 +230,7 @@ const char *_dump_dinput_GUID(const GUID *guid) {
     };
     if (guid == NULL)
 	return "null GUID";
-    for (i = 0; i < (sizeof(guids) / sizeof(guids[0])); i++) {
+    for (i = 0; i < ARRAY_SIZE(guids); i++) {
 	if (IsEqualGUID(guids[i].guid, guid)) {
 	    return guids[i].name;
 	}
@@ -914,7 +916,7 @@ HRESULT _set_action_map(LPDIRECTINPUTDEVICE8W iface, LPDIACTIONFORMATW lpdiaf, L
     if (dwFlags & DIDSAM_NOUSER)
         dps.wsz[0] = '\0';
     else
-        lstrcpynW(dps.wsz, username, sizeof(dps.wsz)/sizeof(WCHAR));
+        lstrcpynW(dps.wsz, username, ARRAY_SIZE(dps.wsz));
     IDirectInputDevice8_SetProperty(iface, DIPROP_USERNAME, &dps.diph);
 
     /* Save the settings to disk */
@@ -935,6 +937,8 @@ void queue_event(LPDIRECTINPUTDEVICE8A iface, int inst_id, DWORD data, DWORD tim
     /* Event is being set regardless of the queue state */
     if (This->hEvent) SetEvent(This->hEvent);
 
+    PostMessageW(GetDesktopWindow(), WM_WINE_NOTIFY_ACTIVITY, 0, 0);
+
     if (!This->queue_len || This->overflow || ofs < 0) return;
 
     next_pos = (This->queue_head + 1) % This->queue_len;
@@ -990,9 +994,9 @@ HRESULT WINAPI IDirectInputDevice2WImpl_Acquire(LPDIRECTINPUTDEVICE8W iface)
     EnterCriticalSection(&This->crit);
     res = This->acquired ? S_FALSE : DI_OK;
     This->acquired = 1;
-    if (res == DI_OK)
-        check_dinput_hooks(iface);
     LeaveCriticalSection(&This->crit);
+    if (res == DI_OK)
+        check_dinput_hooks(iface, TRUE);
 
     return res;
 }
@@ -1018,9 +1022,9 @@ HRESULT WINAPI IDirectInputDevice2WImpl_Unacquire(LPDIRECTINPUTDEVICE8W iface)
     EnterCriticalSection(&This->crit);
     res = !This->acquired ? DI_NOEFFECT : DI_OK;
     This->acquired = 0;
-    if (res == DI_OK)
-        check_dinput_hooks(iface);
     LeaveCriticalSection(&This->crit);
+    if (res == DI_OK)
+        check_dinput_hooks(iface, FALSE);
 
     return res;
 }
@@ -1177,7 +1181,7 @@ HRESULT WINAPI IDirectInputDevice2WImpl_QueryInterface(LPDIRECTINPUTDEVICE8W ifa
 {
     IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
 
-    TRACE("(%p this=%p,%s,%p)\n", iface, This, debugstr_guid(riid), ppobj);
+    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), ppobj);
     if (IsEqualGUID(&IID_IUnknown, riid) ||
         IsEqualGUID(&IID_IDirectInputDeviceA,  riid) ||
         IsEqualGUID(&IID_IDirectInputDevice2A, riid) ||
@@ -1199,7 +1203,7 @@ HRESULT WINAPI IDirectInputDevice2WImpl_QueryInterface(LPDIRECTINPUTDEVICE8W ifa
     }
 
     WARN("Unsupported interface!\n");
-    return E_FAIL;
+    return E_NOINTERFACE;
 }
 
 HRESULT WINAPI IDirectInputDevice2AImpl_QueryInterface(LPDIRECTINPUTDEVICE8A iface, REFIID riid, LPVOID *ppobj)
@@ -1227,7 +1231,7 @@ HRESULT WINAPI IDirectInputDevice2AImpl_EnumObjects(LPDIRECTINPUTDEVICE8A iface,
     DIDEVICEOBJECTINSTANCEA ddoi;
     int i;
 
-    TRACE("(%p) %p,%p flags:%08x)\n", iface, lpCallback, lpvRef, dwFlags);
+    TRACE("(%p)->(%p,%p flags:%08x)\n", This, lpCallback, lpvRef, dwFlags);
     TRACE("  - flags = ");
     _dump_EnumObjects_flags(dwFlags);
     TRACE("\n");
@@ -1257,7 +1261,7 @@ HRESULT WINAPI IDirectInputDevice2WImpl_EnumObjects(LPDIRECTINPUTDEVICE8W iface,
     DIDEVICEOBJECTINSTANCEW ddoi;
     int i;
 
-    TRACE("(%p) %p,%p flags:%08x)\n", iface, lpCallback, lpvRef, dwFlags);
+    TRACE("(%p)->(%p,%p flags:%08x)\n", This, lpCallback, lpvRef, dwFlags);
     TRACE("  - flags = ");
     _dump_EnumObjects_flags(dwFlags);
     TRACE("\n");
@@ -1288,7 +1292,7 @@ HRESULT WINAPI IDirectInputDevice2WImpl_GetProperty(LPDIRECTINPUTDEVICE8W iface,
 {
     IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
 
-    TRACE("(%p) %s,%p\n", iface, debugstr_guid(rguid), pdiph);
+    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(rguid), pdiph);
     _dump_DIPROPHEADER(pdiph);
 
     if (!IS_DIPROP(rguid)) return DI_OK;
@@ -1319,7 +1323,7 @@ HRESULT WINAPI IDirectInputDevice2WImpl_GetProperty(LPDIRECTINPUTDEVICE8W iface,
                 {
                     if (*device_player->username)
                     {
-                        lstrcpynW(ps->wsz, device_player->username, sizeof(ps->wsz)/sizeof(WCHAR));
+                        lstrcpynW(ps->wsz, device_player->username, ARRAY_SIZE(ps->wsz));
                         return DI_OK;
                     }
                     else break;
@@ -1353,7 +1357,7 @@ HRESULT WINAPI IDirectInputDevice2WImpl_SetProperty(
 {
     IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
 
-    TRACE("(%p) %s,%p\n", iface, debugstr_guid(rguid), pdiph);
+    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(rguid), pdiph);
     _dump_DIPROPHEADER(pdiph);
 
     if (!IS_DIPROP(rguid)) return DI_OK;
@@ -1424,8 +1428,7 @@ HRESULT WINAPI IDirectInputDevice2WImpl_SetProperty(
                 device_player->instance_guid = This->guid;
             }
             if (device_player)
-                lstrcpynW(device_player->username, ps->wsz,
-                    sizeof(device_player->username)/sizeof(WCHAR));
+                lstrcpynW(device_player->username, ps->wsz, ARRAY_SIZE(device_player->username));
             break;
         }
         default:
@@ -1593,7 +1596,8 @@ HRESULT WINAPI IDirectInputDevice2AImpl_GetDeviceData(LPDIRECTINPUTDEVICE8A ifac
 
 HRESULT WINAPI IDirectInputDevice2WImpl_RunControlPanel(LPDIRECTINPUTDEVICE8W iface, HWND hwndOwner, DWORD dwFlags)
 {
-    FIXME("(this=%p,%p,0x%08x): stub!\n", iface, hwndOwner, dwFlags);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("%p)->(%p,0x%08x): stub!\n", This, hwndOwner, dwFlags);
 
     return DI_OK;
 }
@@ -1607,7 +1611,8 @@ HRESULT WINAPI IDirectInputDevice2AImpl_RunControlPanel(LPDIRECTINPUTDEVICE8A if
 HRESULT WINAPI IDirectInputDevice2WImpl_Initialize(LPDIRECTINPUTDEVICE8W iface, HINSTANCE hinst, DWORD dwVersion,
                                                    REFGUID rguid)
 {
-    FIXME("(this=%p,%p,%d,%s): stub!\n", iface, hinst, dwVersion, debugstr_guid(rguid));
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%p,%d,%s): stub!\n", This, hinst, dwVersion, debugstr_guid(rguid));
     return DI_OK;
 }
 
@@ -1625,7 +1630,8 @@ HRESULT WINAPI IDirectInputDevice2AImpl_Initialize(LPDIRECTINPUTDEVICE8A iface,
 HRESULT WINAPI IDirectInputDevice2WImpl_CreateEffect(LPDIRECTINPUTDEVICE8W iface, REFGUID rguid, LPCDIEFFECT lpeff,
                                                      LPDIRECTINPUTEFFECT *ppdef, LPUNKNOWN pUnkOuter)
 {
-    FIXME("(this=%p,%s,%p,%p,%p): stub!\n", iface, debugstr_guid(rguid), lpeff, ppdef, pUnkOuter);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%s,%p,%p,%p): stub!\n", This, debugstr_guid(rguid), lpeff, ppdef, pUnkOuter);
 
     FIXME("not available in the generic implementation\n");
     *ppdef = NULL;
@@ -1645,9 +1651,9 @@ HRESULT WINAPI IDirectInputDevice2AImpl_EnumEffects(
 	LPVOID lpvRef,
 	DWORD dwFlags)
 {
-    FIXME("(this=%p,%p,%p,0x%08x): stub!\n",
-	  iface, lpCallback, lpvRef, dwFlags);
-    
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8A(iface);
+    FIXME("%p)->(%p,%p,0x%08x): stub!\n", This, lpCallback, lpvRef, dwFlags);
+
     return DI_OK;
 }
 
@@ -1657,9 +1663,9 @@ HRESULT WINAPI IDirectInputDevice2WImpl_EnumEffects(
 	LPVOID lpvRef,
 	DWORD dwFlags)
 {
-    FIXME("(this=%p,%p,%p,0x%08x): stub!\n",
-	  iface, lpCallback, lpvRef, dwFlags);
-    
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%p,%p,0x%08x): stub!\n", This, lpCallback, lpvRef, dwFlags);
+
     return DI_OK;
 }
 
@@ -1668,8 +1674,8 @@ HRESULT WINAPI IDirectInputDevice2AImpl_GetEffectInfo(
 	LPDIEFFECTINFOA lpdei,
 	REFGUID rguid)
 {
-    FIXME("(this=%p,%p,%s): stub!\n",
-	  iface, lpdei, debugstr_guid(rguid));
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8A(iface);
+    FIXME("(%p)->(%p,%s): stub!\n", This, lpdei, debugstr_guid(rguid));
     return DI_OK;
 }
 
@@ -1678,14 +1684,15 @@ HRESULT WINAPI IDirectInputDevice2WImpl_GetEffectInfo(
 	LPDIEFFECTINFOW lpdei,
 	REFGUID rguid)
 {
-    FIXME("(this=%p,%p,%s): stub!\n",
-	  iface, lpdei, debugstr_guid(rguid));
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%p,%s): stub!\n", This, lpdei, debugstr_guid(rguid));
     return DI_OK;
 }
 
 HRESULT WINAPI IDirectInputDevice2WImpl_GetForceFeedbackState(LPDIRECTINPUTDEVICE8W iface, LPDWORD pdwOut)
 {
-    FIXME("(this=%p,%p): stub!\n", iface, pdwOut);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%p): stub!\n", This, pdwOut);
     return DI_OK;
 }
 
@@ -1697,7 +1704,8 @@ HRESULT WINAPI IDirectInputDevice2AImpl_GetForceFeedbackState(LPDIRECTINPUTDEVIC
 
 HRESULT WINAPI IDirectInputDevice2WImpl_SendForceFeedbackCommand(LPDIRECTINPUTDEVICE8W iface, DWORD dwFlags)
 {
-    TRACE("(%p) 0x%08x:\n", iface, dwFlags);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    TRACE("(%p)->(0x%08x)\n", This, dwFlags);
     return DI_NOEFFECT;
 }
 
@@ -1710,7 +1718,8 @@ HRESULT WINAPI IDirectInputDevice2AImpl_SendForceFeedbackCommand(LPDIRECTINPUTDE
 HRESULT WINAPI IDirectInputDevice2WImpl_EnumCreatedEffectObjects(LPDIRECTINPUTDEVICE8W iface,
         LPDIENUMCREATEDEFFECTOBJECTSCALLBACK lpCallback, LPVOID lpvRef, DWORD dwFlags)
 {
-    FIXME("(this=%p,%p,%p,0x%08x): stub!\n", iface, lpCallback, lpvRef, dwFlags);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)0>(%p,%p,0x%08x): stub!\n", This, lpCallback, lpvRef, dwFlags);
     return DI_OK;
 }
 
@@ -1723,7 +1732,8 @@ HRESULT WINAPI IDirectInputDevice2AImpl_EnumCreatedEffectObjects(LPDIRECTINPUTDE
 
 HRESULT WINAPI IDirectInputDevice2WImpl_Escape(LPDIRECTINPUTDEVICE8W iface, LPDIEFFESCAPE lpDIEEsc)
 {
-    FIXME("(this=%p,%p): stub!\n", iface, lpDIEEsc);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%p): stub!\n", This, lpDIEEsc);
     return DI_OK;
 }
 
@@ -1753,7 +1763,8 @@ HRESULT WINAPI IDirectInputDevice2WImpl_SendDeviceData(LPDIRECTINPUTDEVICE8W ifa
                                                        LPCDIDEVICEOBJECTDATA rgdod, LPDWORD pdwInOut,
                                                        DWORD dwFlags)
 {
-    FIXME("(this=%p,0x%08x,%p,%p,0x%08x): stub!\n", iface, cbObjectData, rgdod, pdwInOut, dwFlags);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(0x%08x,%p,%p,0x%08x): stub!\n", This, cbObjectData, rgdod, pdwInOut, dwFlags);
 
     return DI_OK;
 }
@@ -1773,7 +1784,8 @@ HRESULT WINAPI IDirectInputDevice7AImpl_EnumEffectsInFile(LPDIRECTINPUTDEVICE8A
 							  LPVOID pvRef,
 							  DWORD dwFlags)
 {
-    FIXME("(%p)->(%s,%p,%p,%08x): stub !\n", iface, lpszFileName, pec, pvRef, dwFlags);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8A(iface);
+    FIXME("(%p)->(%s,%p,%p,%08x): stub !\n", This, lpszFileName, pec, pvRef, dwFlags);
     
     return DI_OK;
 }
@@ -1784,7 +1796,8 @@ HRESULT WINAPI IDirectInputDevice7WImpl_EnumEffectsInFile(LPDIRECTINPUTDEVICE8W
 							  LPVOID pvRef,
 							  DWORD dwFlags)
 {
-    FIXME("(%p)->(%s,%p,%p,%08x): stub !\n", iface, debugstr_w(lpszFileName), pec, pvRef, dwFlags);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%s,%p,%p,%08x): stub !\n", This, debugstr_w(lpszFileName), pec, pvRef, dwFlags);
     
     return DI_OK;
 }
@@ -1795,7 +1808,8 @@ HRESULT WINAPI IDirectInputDevice7AImpl_WriteEffectToFile(LPDIRECTINPUTDEVICE8A
 							  LPDIFILEEFFECT rgDiFileEft,
 							  DWORD dwFlags)
 {
-    FIXME("(%p)->(%s,%08x,%p,%08x): stub !\n", iface, lpszFileName, dwEntries, rgDiFileEft, dwFlags);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8A(iface);
+    FIXME("(%p)->(%s,%08x,%p,%08x): stub !\n", This, lpszFileName, dwEntries, rgDiFileEft, dwFlags);
     
     return DI_OK;
 }
@@ -1806,7 +1820,8 @@ HRESULT WINAPI IDirectInputDevice7WImpl_WriteEffectToFile(LPDIRECTINPUTDEVICE8W
 							  LPDIFILEEFFECT rgDiFileEft,
 							  DWORD dwFlags)
 {
-    FIXME("(%p)->(%s,%08x,%p,%08x): stub !\n", iface, debugstr_w(lpszFileName), dwEntries, rgDiFileEft, dwFlags);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%s,%08x,%p,%08x): stub !\n", This, debugstr_w(lpszFileName), dwEntries, rgDiFileEft, dwFlags);
     
     return DI_OK;
 }
@@ -1816,7 +1831,8 @@ HRESULT WINAPI IDirectInputDevice8WImpl_BuildActionMap(LPDIRECTINPUTDEVICE8W ifa
 						       LPCWSTR lpszUserName,
 						       DWORD dwFlags)
 {
-    FIXME("(%p)->(%p,%s,%08x): semi-stub !\n", iface, lpdiaf, debugstr_w(lpszUserName), dwFlags);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%p,%s,%08x): semi-stub !\n", This, lpdiaf, debugstr_w(lpszUserName), dwFlags);
 #define X(x) if (dwFlags & x) FIXME("\tdwFlags =|"#x"\n");
 	X(DIDBAM_DEFAULT)
 	X(DIDBAM_PRESERVE)
@@ -1830,7 +1846,8 @@ HRESULT WINAPI IDirectInputDevice8WImpl_BuildActionMap(LPDIRECTINPUTDEVICE8W ifa
 HRESULT WINAPI IDirectInputDevice8AImpl_GetImageInfo(LPDIRECTINPUTDEVICE8A iface,
 						     LPDIDEVICEIMAGEINFOHEADERA lpdiDevImageInfoHeader)
 {
-    FIXME("(%p)->(%p): stub !\n", iface, lpdiDevImageInfoHeader);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8A(iface);
+    FIXME("(%p)->(%p): stub !\n", This, lpdiDevImageInfoHeader);
     
     return DI_OK;
 }
@@ -1838,7 +1855,8 @@ HRESULT WINAPI IDirectInputDevice8AImpl_GetImageInfo(LPDIRECTINPUTDEVICE8A iface
 HRESULT WINAPI IDirectInputDevice8WImpl_GetImageInfo(LPDIRECTINPUTDEVICE8W iface,
 						     LPDIDEVICEIMAGEINFOHEADERW lpdiDevImageInfoHeader)
 {
-    FIXME("(%p)->(%p): stub !\n", iface, lpdiDevImageInfoHeader);
+    IDirectInputDeviceImpl *This = impl_from_IDirectInputDevice8W(iface);
+    FIXME("(%p)->(%p): stub !\n", This, lpdiDevImageInfoHeader);
     
     return DI_OK;
 }
