@@ -23,6 +23,7 @@
 #include <stdarg.h>
 #include "hid.h"
 #include "ddk/hidtypes.h"
+#include "ddk/wdm.h"
 #include "regstr.h"
 #include "wine/debug.h"
 #include "wine/unicode.h"
@@ -75,6 +76,7 @@ static NTSTATUS get_device_id(DEVICE_OBJECT *device, BUS_QUERY_ID_TYPE type, WCH
 
 NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
 {
+    hid_device *hiddev;
     DEVICE_OBJECT *device = NULL;
     NTSTATUS status;
     minidriver *minidriver;
@@ -96,16 +98,23 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
     TRACE("PDO add device(%p:%s)\n", PDO, debugstr_w(PDO_id));
     minidriver = find_minidriver(driver);
 
-    status = HID_CreateDevice(PDO, &minidriver->minidriver, &device);
+    hiddev = HeapAlloc(GetProcessHeap(), 0, sizeof(*hiddev));
+    if (!hiddev)
+        return STATUS_NO_MEMORY;
+
+    status = HID_CreateDevice(PDO, &minidriver->minidriver, &hiddev->device);
     if (status != STATUS_SUCCESS)
     {
         ERR("Failed to create HID object (%x)\n",status);
         HeapFree(GetProcessHeap(), 0, PDO_id);
+        HeapFree(GetProcessHeap(), 0, hiddev);
         return status;
     }
+    device = hiddev->device;
 
     ext = device->DeviceExtension;
     InitializeListHead(&ext->irp_queue);
+    KeInitializeSpinLock(&ext->irp_queue_lock);
 
     TRACE("Created device %p\n",device);
     status = minidriver->AddDevice(minidriver->minidriver.DriverObject, device);
@@ -177,6 +186,8 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
         return STATUS_NOT_SUPPORTED;
     }
 
+    list_add_tail(&(minidriver->device_list), &hiddev->entry);
+
     ext->information.DescriptorSize = ext->preparseData->dwSize;
 
     lstrcpyW(ext->instance_id, device_enumeratorW);
@@ -200,6 +211,34 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
     return STATUS_SUCCESS;
 }
 
+NTSTATUS PNP_RemoveDevice(minidriver *minidriver, DEVICE_OBJECT *device, IRP *irp)
+{
+    BASE_DEVICE_EXTENSION *ext = device->DeviceExtension;
+    hid_device *hiddev;
+    NTSTATUS rc = STATUS_NOT_SUPPORTED;
+
+    rc = IoSetDeviceInterfaceState(&ext->link_name, FALSE);
+    if (rc)
+    {
+        FIXME("failed to disable interface %x\n", rc);
+        return rc;
+    }
+
+    if (irp)
+        rc = minidriver->PNPDispatch(device, irp);
+    HID_DeleteDevice(&minidriver->minidriver, device);
+    LIST_FOR_EACH_ENTRY(hiddev,  &minidriver->device_list, hid_device, entry)
+    {
+        if (hiddev->device == device)
+        {
+            list_remove(&hiddev->entry);
+            HeapFree(GetProcessHeap(), 0, hiddev);
+            break;
+        }
+    }
+    return rc;
+}
+
 NTSTATUS WINAPI HID_PNP_Dispatch(DEVICE_OBJECT *device, IRP *irp)
 {
     NTSTATUS rc = STATUS_NOT_SUPPORTED;
@@ -253,11 +292,18 @@ NTSTATUS WINAPI HID_PNP_Dispatch(DEVICE_OBJECT *device, IRP *irp)
             }
             break;
         }
+        case IRP_MN_START_DEVICE:
+        {
+            BASE_DEVICE_EXTENSION *ext = device->DeviceExtension;
+
+            rc = minidriver->PNPDispatch(device, irp);
+
+            IoSetDeviceInterfaceState(&ext->link_name, TRUE);
+            return rc;
+        }
         case IRP_MN_REMOVE_DEVICE:
         {
-            rc = minidriver->PNPDispatch(device, irp);
-            HID_DeleteDevice(&minidriver->minidriver, device);
-            return rc;
+            return PNP_RemoveDevice(minidriver, device, irp);
         }
         default:
         {
