@@ -137,7 +137,8 @@ static void destroy_joysticks(struct JoystickData *data)
         if (data->joysticks[i].forcefeedback && data->joysticks[i].num_effects > 0)
         {
             for (j = 0; j < data->joysticks[i].num_effects; j++)
-                IDirectInputEffect_Release(data->joysticks[i].effects[j].effect);
+                if (data->joysticks[i].effects[j].effect)
+                    IDirectInputEffect_Release(data->joysticks[i].effects[j].effect);
 
             HeapFree(GetProcessHeap(), 0, data->joysticks[i].effects);
         }
@@ -416,7 +417,7 @@ static DWORD WINAPI input_thread(void *param)
         axes_pos[2][1] = state.lRz;
 
         /* Set pov values */
-        for (j = 0; j < sizeof(pov_val)/sizeof(pov_val[0]); j++)
+        for (j = 0; j < ARRAY_SIZE(pov_val); j++)
         {
             if (state.rgdwPOV[0] == pov_val[j])
             {
@@ -677,6 +678,8 @@ static DWORD WINAPI ff_input_thread(void *param)
         DWORD flags = DIEP_AXES | DIEP_DIRECTION | DIEP_NORESTART;
         RECT r;
 
+        Sleep(TEST_POLL_TIME);
+
         /* Skip this if we have no effects */
         if (joy->num_effects == 0 || chosen_effect < 0) continue;
 
@@ -701,8 +704,6 @@ static DWORD WINAPI ff_input_thread(void *param)
                 IDirectInputEffect_Start(joy->effects[chosen_effect].effect, 1, 0);
                 break;
             }
-
-        Sleep(TEST_POLL_TIME);
     }
 
     return 0;
