@@ -48,7 +48,7 @@ typedef struct {
     int ref;
 } function_local_t;
 
-typedef struct {
+typedef struct _compiler_ctx_t {
     parser_ctx_t *parser;
     bytecode_t *code;
 
@@ -130,7 +130,7 @@ static inline void *compiler_alloc(bytecode_t *code, size_t size)
     return heap_pool_alloc(&code->heap, size);
 }
 
-static jsstr_t *compiler_alloc_string_len(compiler_ctx_t *ctx, const WCHAR *str, unsigned len)
+jsstr_t *compiler_alloc_string_len(compiler_ctx_t *ctx, const WCHAR *str, unsigned len)
 {
     jsstr_t *new_str;
 
@@ -245,14 +245,9 @@ static HRESULT push_instr_int(compiler_ctx_t *ctx, jsop_t op, LONG arg)
     return S_OK;
 }
 
-static HRESULT push_instr_str(compiler_ctx_t *ctx, jsop_t op, const WCHAR *arg)
+static HRESULT push_instr_str(compiler_ctx_t *ctx, jsop_t op, jsstr_t *str)
 {
     unsigned instr;
-    jsstr_t *str;
-
-    str = compiler_alloc_string(ctx, arg);
-    if(!str)
-        return E_OUTOFMEMORY;
 
     instr = push_instr(ctx, op);
     if(!instr)
@@ -262,6 +257,19 @@ static HRESULT push_instr_str(compiler_ctx_t *ctx, jsop_t op, const WCHAR *arg)
     return S_OK;
 }
 
+static HRESULT push_instr_str_uint(compiler_ctx_t *ctx, jsop_t op, jsstr_t *str, unsigned arg2)
+{
+    unsigned instr;
+
+    instr = push_instr(ctx, op);
+    if(!instr)
+        return E_OUTOFMEMORY;
+
+    instr_ptr(ctx, instr)->u.arg[0].str = str;
+    instr_ptr(ctx, instr)->u.arg[1].uint = arg2;
+    return S_OK;
+}
+
 static HRESULT push_instr_bstr(compiler_ctx_t *ctx, jsop_t op, const WCHAR *arg)
 {
     unsigned instr;
@@ -477,13 +485,18 @@ static HRESULT compile_memberid_expression(compiler_ctx_t *ctx, expression_t *ex
     }
     case EXPR_MEMBER: {
         member_expression_t *member_expr = (member_expression_t*)expr;
+        jsstr_t *jsstr;
 
         hres = compile_expression(ctx, member_expr->expression, TRUE);
         if(FAILED(hres))
             return hres;
 
         /* FIXME: Potential optimization */
-        hres = push_instr_str(ctx, OP_str, member_expr->identifier);
+        jsstr = compiler_alloc_string(ctx, member_expr->identifier);
+        if(!jsstr)
+            return E_OUTOFMEMORY;
+
+        hres = push_instr_str(ctx, OP_str, jsstr);
         if(FAILED(hres))
             return hres;
 
@@ -676,13 +689,18 @@ static HRESULT compile_delete_expression(compiler_ctx_t *ctx, unary_expression_t
     }
     case EXPR_MEMBER: {
         member_expression_t *member_expr = (member_expression_t*)expr->expression;
+        jsstr_t *jsstr;
 
         hres = compile_expression(ctx, member_expr->expression, TRUE);
         if(FAILED(hres))
             return hres;
 
         /* FIXME: Potential optimization */
-        hres = push_instr_str(ctx, OP_str, member_expr->identifier);
+        jsstr = compiler_alloc_string(ctx, member_expr->identifier);
+        if(!jsstr)
+            return E_OUTOFMEMORY;
+
+        hres = push_instr_str(ctx, OP_str, jsstr);
         if(FAILED(hres))
             return hres;
 
@@ -811,48 +829,22 @@ static HRESULT compile_literal(compiler_ctx_t *ctx, literal_t *literal)
     case LT_NULL:
         return push_instr(ctx, OP_null) ? S_OK : E_OUTOFMEMORY;
     case LT_STRING:
-        return push_instr_str(ctx, OP_str, literal->u.wstr);
-    case LT_REGEXP: {
-        unsigned instr;
-        jsstr_t *str;
-
-        str = compiler_alloc_string_len(ctx, literal->u.regexp.str, literal->u.regexp.str_len);
-        if(!str)
-            return E_OUTOFMEMORY;
-
-        instr = push_instr(ctx, OP_regexp);
-        if(!instr)
-            return E_OUTOFMEMORY;
-
-        instr_ptr(ctx, instr)->u.arg[0].str = str;
-        instr_ptr(ctx, instr)->u.arg[1].uint = literal->u.regexp.flags;
-        return S_OK;
-    }
+        return push_instr_str(ctx, OP_str, literal->u.str);
+    case LT_REGEXP:
+        return push_instr_str_uint(ctx, OP_regexp, literal->u.regexp.str, literal->u.regexp.flags);
     DEFAULT_UNREACHABLE;
     }
     return E_FAIL;
 }
 
-static HRESULT literal_as_bstr(compiler_ctx_t *ctx, literal_t *literal, BSTR *str)
+static HRESULT literal_as_string(compiler_ctx_t *ctx, literal_t *literal, jsstr_t **str)
 {
     switch(literal->type) {
     case LT_STRING:
-        *str = compiler_alloc_bstr(ctx, literal->u.wstr);
+        *str = literal->u.str;
         break;
-    case LT_DOUBLE: {
-        jsstr_t *jsstr;
-        HRESULT hres;
-
-        hres = double_to_string(literal->u.dval, &jsstr);
-        if(FAILED(hres))
-            return hres;
-
-        *str = compiler_alloc_bstr_len(ctx, NULL, jsstr_length(jsstr));
-        if(*str)
-            jsstr_flush(jsstr, *str);
-        jsstr_release(jsstr);
-        break;
-    }
+    case LT_DOUBLE:
+        return double_to_string(literal->u.dval, str);
     DEFAULT_UNREACHABLE;
     }
 
@@ -889,14 +881,14 @@ static HRESULT compile_array_literal(compiler_ctx_t *ctx, array_literal_expressi
 static HRESULT compile_object_literal(compiler_ctx_t *ctx, property_value_expression_t *expr)
 {
     property_definition_t *iter;
-    BSTR name;
+    jsstr_t *name;
     HRESULT hres;
 
     if(!push_instr(ctx, OP_new_obj))
         return E_OUTOFMEMORY;
 
     for(iter = expr->property_list; iter; iter = iter->next) {
-        hres = literal_as_bstr(ctx, iter->name, &name);
+        hres = literal_as_string(ctx, iter->name, &name);
         if(FAILED(hres))
             return hres;
 
@@ -904,7 +896,7 @@ static HRESULT compile_object_literal(compiler_ctx_t *ctx, property_value_expres
         if(FAILED(hres))
             return hres;
 
-        hres = push_instr_bstr_uint(ctx, OP_obj_prop, name, iter->type);
+        hres = push_instr_str_uint(ctx, OP_obj_prop, name, iter->type);
         if(FAILED(hres))
             return hres;
     }
@@ -2479,7 +2471,7 @@ HRESULT compile_script(script_ctx_t *ctx, const WCHAR *code, const WCHAR *args,
         }
     }
 
-    hres = script_parse(ctx, compiler.code->source, delimiter, from_eval, &compiler.parser);
+    hres = script_parse(ctx, &compiler, compiler.code->source, delimiter, from_eval, &compiler.parser);
     if(FAILED(hres)) {
         release_bytecode(compiler.code);
         return hres;
