@@ -32,8 +32,11 @@ static const WCHAR propertyIsEnumerableW[] =
     {'p','r','o','p','e','r','t','y','I','s','E','n','u','m','e','r','a','b','l','e',0};
 static const WCHAR isPrototypeOfW[] = {'i','s','P','r','o','t','o','t','y','p','e','O','f',0};
 
+static const WCHAR createW[] = {'c','r','e','a','t','e',0};
 static const WCHAR getOwnPropertyDescriptorW[] =
     {'g','e','t','O','w','n','P','r','o','p','e','r','t','y','D','e','s','c','r','i','p','t','o','r',0};
+static const WCHAR getPrototypeOfW[] =
+    {'g','e','t','P','r','o','t','o','t','y','p','e','O','f',0};
 static const WCHAR definePropertyW[] = {'d','e','f','i','n','e','P','r','o','p','e','r','t','y',0};
 
 static const WCHAR definePropertiesW[] = {'d','e','f','i','n','e','P','r','o','p','e','r','t','i','e','s',0};
@@ -66,7 +69,7 @@ static HRESULT Object_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     static const WCHAR regexpW[] = {'R','e','g','E','x','p',0};
     static const WCHAR stringW[] = {'S','t','r','i','n','g',0};
     /* Keep in sync with jsclass_t enum */
-    static const WCHAR *names[] = {NULL, arrayW, booleanW, dateW, errorW,
+    static const WCHAR *names[] = {NULL, arrayW, booleanW, dateW, objectW, errorW,
         functionW, NULL, mathW, numberW, objectW, regexpW, stringW, objectW, objectW, objectW};
 
     TRACE("\n");
@@ -252,7 +255,7 @@ static const builtin_prop_t Object_props[] = {
 static const builtin_info_t Object_info = {
     JSCLASS_OBJECT,
     {NULL, NULL,0, Object_get_value},
-    sizeof(Object_props)/sizeof(*Object_props),
+    ARRAY_SIZE(Object_props),
     Object_props,
     Object_destructor,
     NULL
@@ -493,23 +496,23 @@ static HRESULT Object_getOwnPropertyDescriptor(script_ctx_t *ctx, vdisp_t *jsthi
         return hres;
 
     if(prop_desc.explicit_getter || prop_desc.explicit_setter) {
-        hres = jsdisp_propput_name(desc_obj, getW, prop_desc.getter
-                                   ? jsval_obj(prop_desc.getter) : jsval_undefined());
+        hres = jsdisp_define_data_property(desc_obj, getW, PROPF_ALL,
+                prop_desc.getter ? jsval_obj(prop_desc.getter) : jsval_undefined());
         if(SUCCEEDED(hres))
-            hres = jsdisp_propput_name(desc_obj, setW, prop_desc.setter
-                                       ? jsval_obj(prop_desc.setter) : jsval_undefined());
+            hres = jsdisp_define_data_property(desc_obj, setW, PROPF_ALL,
+                    prop_desc.setter ? jsval_obj(prop_desc.setter) : jsval_undefined());
     }else {
         hres = jsdisp_propput_name(desc_obj, valueW, prop_desc.value);
         if(SUCCEEDED(hres))
-            hres = jsdisp_propput_name(desc_obj, writableW,
-                                       jsval_bool(!!(prop_desc.flags & PROPF_WRITABLE)));
+            hres = jsdisp_define_data_property(desc_obj, writableW, PROPF_ALL,
+                    jsval_bool(!!(prop_desc.flags & PROPF_WRITABLE)));
     }
     if(SUCCEEDED(hres))
-        hres = jsdisp_propput_name(desc_obj, enumerableW,
-                                   jsval_bool(!!(prop_desc.flags & PROPF_ENUMERABLE)));
+        hres = jsdisp_define_data_property(desc_obj, enumerableW, PROPF_ALL,
+                jsval_bool(!!(prop_desc.flags & PROPF_ENUMERABLE)));
     if(SUCCEEDED(hres))
-        hres = jsdisp_propput_name(desc_obj, configurableW,
-                                   jsval_bool(!!(prop_desc.flags & PROPF_CONFIGURABLE)));
+        hres = jsdisp_define_data_property(desc_obj, configurableW, PROPF_ALL,
+                jsval_bool(!!(prop_desc.flags & PROPF_CONFIGURABLE)));
 
     release_property_descriptor(&prop_desc);
     if(SUCCEEDED(hres) && r)
@@ -519,16 +522,82 @@ static HRESULT Object_getOwnPropertyDescriptor(script_ctx_t *ctx, vdisp_t *jsthi
     return hres;
 }
 
+static HRESULT Object_create(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
+                             unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    jsdisp_t *proto = NULL, *obj;
+    HRESULT hres;
+
+    if(!argc || (!is_object_instance(argv[0]) && !is_null(argv[0]))) {
+        FIXME("Invalid arg\n");
+        return E_INVALIDARG;
+    }
+
+    TRACE("(%s)\n", debugstr_jsval(argv[0]));
+
+    if(argc > 1) {
+        FIXME("Unsupported properties argument %s\n", debugstr_jsval(argv[1]));
+        return E_NOTIMPL;
+    }
+
+    if(argc && is_object_instance(argv[0])) {
+        if(get_object(argv[0]))
+            proto = to_jsdisp(get_object(argv[0]));
+        if(!proto) {
+            FIXME("Non-JS prototype\n");
+            return E_NOTIMPL;
+        }
+    }else if(!is_null(argv[0])) {
+        FIXME("Invalid arg %s\n", debugstr_jsval(argc ? argv[0] : jsval_undefined()));
+        return E_INVALIDARG;
+    }
+
+    if(r) {
+        hres = create_dispex(ctx, NULL, proto, &obj);
+        if(FAILED(hres))
+            return hres;
+        *r = jsval_obj(obj);
+    }
+    return S_OK;
+}
+
+static HRESULT Object_getPrototypeOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
+                                     unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    jsdisp_t *obj;
+
+    if(!argc || !is_object_instance(argv[0])) {
+        FIXME("invalid arguments\n");
+        return E_NOTIMPL;
+    }
+
+    TRACE("(%s)\n", debugstr_jsval(argv[1]));
+
+    obj = to_jsdisp(get_object(argv[0]));
+    if(!obj) {
+        FIXME("Non-JS object\n");
+        return E_NOTIMPL;
+    }
+
+    if(r)
+        *r = obj->prototype
+            ? jsval_obj(jsdisp_addref(obj->prototype))
+            : jsval_null();
+    return S_OK;
+}
+
 static const builtin_prop_t ObjectConstr_props[] = {
+    {createW,                   Object_create,                      PROPF_ES5|PROPF_METHOD|2},
     {definePropertiesW,         Object_defineProperties,            PROPF_ES5|PROPF_METHOD|2},
     {definePropertyW,           Object_defineProperty,              PROPF_ES5|PROPF_METHOD|2},
-    {getOwnPropertyDescriptorW, Object_getOwnPropertyDescriptor,    PROPF_ES5|PROPF_METHOD|2}
+    {getOwnPropertyDescriptorW, Object_getOwnPropertyDescriptor,    PROPF_ES5|PROPF_METHOD|2},
+    {getPrototypeOfW,           Object_getPrototypeOf,              PROPF_ES5|PROPF_METHOD|1}
 };
 
 static const builtin_info_t ObjectConstr_info = {
     JSCLASS_FUNCTION,
     DEFAULT_FUNCTION_VALUE,
-    sizeof(ObjectConstr_props)/sizeof(*ObjectConstr_props),
+    ARRAY_SIZE(ObjectConstr_props),
     ObjectConstr_props,
     NULL,
     NULL
