@@ -53,12 +53,19 @@ WINE_DEFAULT_DEBUG_CHANNEL(time);
 
 #define CALINFO_MAX_YEAR 2029
 
-#define LL2FILETIME( ll, pft )\
-    (pft)->dwLowDateTime = (UINT)(ll); \
-    (pft)->dwHighDateTime = (UINT)((ll) >> 32);
-#define FILETIME2LL( pft, ll) \
-    ll = (((LONGLONG)((pft)->dwHighDateTime))<<32) + (pft)-> dwLowDateTime ;
+static inline void longlong_to_filetime( LONGLONG t, FILETIME *ft )
+{
+    ft->dwLowDateTime = (DWORD)t;
+    ft->dwHighDateTime = (DWORD)(t >> 32);
+}
 
+static inline LONGLONG filetime_to_longlong( const FILETIME *ft )
+{
+    return (((LONGLONG)ft->dwHighDateTime) << 32) + ft->dwLowDateTime;
+}
+
+static const WCHAR mui_stdW[] = { 'M','U','I','_','S','t','d',0 };
+static const WCHAR mui_dltW[] = { 'M','U','I','_','D','l','t',0 };
 
 static const int MonthLengths[2][12] =
 {
@@ -139,7 +146,7 @@ static int TIME_DayLightCompareDate( const SYSTEMTIME *date,
  *
  *  PARAMS
  *      pTZinfo     [in] The time zone data.
- *      lpFileTime  [in] The system or local time.
+ *      time        [in] The system or local time.
  *      islocal     [in] it is local time.
  *
  *  RETURNS
@@ -149,14 +156,13 @@ static int TIME_DayLightCompareDate( const SYSTEMTIME *date,
  *      TIME_ZONE_ID_DAYLIGHT   Current time is daylight savings time
  */
 static DWORD TIME_CompTimeZoneID ( const TIME_ZONE_INFORMATION *pTZinfo,
-    FILETIME *lpFileTime, BOOL islocal )
+                                   LONGLONG time, BOOL islocal )
 {
     int ret, year;
     BOOL beforeStandardDate, afterDaylightDate;
     DWORD retval = TIME_ZONE_ID_INVALID;
-    LONGLONG llTime = 0; /* initialized to prevent gcc complaining */
     SYSTEMTIME SysTime;
-    FILETIME ftTemp;
+    FILETIME ft;
 
     if (pTZinfo->DaylightDate.wMonth != 0)
     {
@@ -174,20 +180,17 @@ static DWORD TIME_CompTimeZoneID ( const TIME_ZONE_INFORMATION *pTZinfo,
             return TIME_ZONE_ID_INVALID;
         }
 
-        if (!islocal) {
-            FILETIME2LL( lpFileTime, llTime );
-            llTime -= pTZinfo->Bias * (LONGLONG)600000000;
-            LL2FILETIME( llTime, &ftTemp)
-            lpFileTime = &ftTemp;
-        }
+        if (!islocal)
+            time -= pTZinfo->Bias * (LONGLONG)600000000;
 
-        FileTimeToSystemTime(lpFileTime, &SysTime);
+        longlong_to_filetime( time, &ft );
+        FileTimeToSystemTime( &ft, &SysTime );
         year = SysTime.wYear;
 
         if (!islocal) {
-            llTime -= pTZinfo->DaylightBias * (LONGLONG)600000000;
-            LL2FILETIME( llTime, &ftTemp)
-            FileTimeToSystemTime(lpFileTime, &SysTime);
+            time -= pTZinfo->DaylightBias * (LONGLONG)600000000;
+            longlong_to_filetime( time, &ft );
+            FileTimeToSystemTime( &ft, &SysTime );
         }
 
         /* check for daylight savings */
@@ -201,14 +204,13 @@ static DWORD TIME_CompTimeZoneID ( const TIME_ZONE_INFORMATION *pTZinfo,
             beforeStandardDate = SysTime.wYear < year;
 
         if (!islocal) {
-            llTime -= ( pTZinfo->StandardBias - pTZinfo->DaylightBias )
-                * (LONGLONG)600000000;
-            LL2FILETIME( llTime, &ftTemp)
-            FileTimeToSystemTime(lpFileTime, &SysTime);
+            time -= ( pTZinfo->StandardBias - pTZinfo->DaylightBias ) * (LONGLONG)600000000;
+            longlong_to_filetime( time, &ft );
+            FileTimeToSystemTime( &ft, &SysTime );
         }
 
         if(year == SysTime.wYear) {
-            ret = TIME_DayLightCompareDate( &SysTime, &pTZinfo->DaylightDate);
+            ret = TIME_DayLightCompareDate( &SysTime, &pTZinfo->DaylightDate );
             if (ret == -2)
                 return TIME_ZONE_ID_INVALID;
 
@@ -217,7 +219,7 @@ static DWORD TIME_CompTimeZoneID ( const TIME_ZONE_INFORMATION *pTZinfo,
             afterDaylightDate = SysTime.wYear > year;
 
         retval = TIME_ZONE_ID_STANDARD;
-        if( pTZinfo->DaylightDate.wMonth <  pTZinfo->StandardDate.wMonth ) {
+        if( pTZinfo->DaylightDate.wMonth < pTZinfo->StandardDate.wMonth ) {
             /* Northern hemisphere */
             if( beforeStandardDate && afterDaylightDate )
                 retval = TIME_ZONE_ID_DAYLIGHT;
@@ -247,9 +249,10 @@ static DWORD TIME_CompTimeZoneID ( const TIME_ZONE_INFORMATION *pTZinfo,
  */
 static DWORD TIME_ZoneID( const TIME_ZONE_INFORMATION *pTzi )
 {
-    FILETIME ftTime;
-    GetSystemTimeAsFileTime( &ftTime);
-    return TIME_CompTimeZoneID( pTzi, &ftTime, FALSE);
+    LARGE_INTEGER now;
+
+    NtQuerySystemTime( &now );
+    return TIME_CompTimeZoneID( pTzi, now.QuadPart, FALSE );
 }
 
 /***********************************************************************
@@ -259,7 +262,7 @@ static DWORD TIME_ZoneID( const TIME_ZONE_INFORMATION *pTzi )
  *
  * PARAMS
  *  pTZinfo    [in]  The time zone data.
- *  lpFileTime [in]  The system or local time.
+ *  time       [in]  The system or local time.
  *  islocal    [in]  It is local time.
  *  pBias      [out] The calculated bias in minutes.
  *
@@ -267,10 +270,10 @@ static DWORD TIME_ZoneID( const TIME_ZONE_INFORMATION *pTzi )
  *  TRUE when the time zone bias was calculated.
  */
 static BOOL TIME_GetTimezoneBias( const TIME_ZONE_INFORMATION *pTZinfo,
-    FILETIME *lpFileTime, BOOL islocal, LONG *pBias )
+                                  LONGLONG time, BOOL islocal, LONG *pBias )
 {
     LONG bias = pTZinfo->Bias;
-    DWORD tzid = TIME_CompTimeZoneID( pTZinfo, lpFileTime, islocal);
+    DWORD tzid = TIME_CompTimeZoneID( pTZinfo, time, islocal );
 
     if( tzid == TIME_ZONE_ID_INVALID)
         return FALSE;
@@ -367,6 +370,24 @@ static BOOL reg_query_value(HKEY hkey, LPCWSTR name, DWORD type, void *data, DWO
     return TRUE;
 }
 
+static BOOL reg_load_mui_string(HKEY hkey, LPCWSTR value, LPWSTR buffer, DWORD size)
+{
+    static const WCHAR advapi32W[] = {'a','d','v','a','p','i','3','2','.','d','l','l',0};
+    DWORD (WINAPI *pRegLoadMUIStringW)(HKEY, LPCWSTR, LPWSTR, DWORD, DWORD *, DWORD, LPCWSTR);
+    HMODULE hDll;
+    BOOL ret = FALSE;
+
+    hDll = LoadLibraryExW(advapi32W, NULL, LOAD_LIBRARY_SEARCH_SYSTEM32);
+    if (hDll) {
+        pRegLoadMUIStringW = (void *)GetProcAddress(hDll, "RegLoadMUIStringW");
+        if (pRegLoadMUIStringW &&
+            !pRegLoadMUIStringW(hkey, value, buffer, size, NULL, 0, DIR_System))
+            ret = TRUE;
+        FreeLibrary(hDll);
+    }
+    return ret;
+}
+
 /***********************************************************************
  *  TIME_GetSpecificTimeZoneInfo
  *
@@ -406,8 +427,15 @@ static BOOL TIME_GetSpecificTimeZoneInfo( const WCHAR *key_name, WORD year,
     if (!TIME_GetSpecificTimeZoneKey( key_name, &time_zone_key ))
         return FALSE;
 
-    if (!reg_query_value( time_zone_key, stdW, REG_SZ, tzinfo->StandardName, sizeof(tzinfo->StandardName)) ||
-        !reg_query_value( time_zone_key, dltW, REG_SZ, tzinfo->DaylightName, sizeof(tzinfo->DaylightName)))
+    if (!reg_load_mui_string( time_zone_key, mui_stdW, tzinfo->StandardName, sizeof(tzinfo->StandardName) ) &&
+        !reg_query_value( time_zone_key, stdW, REG_SZ, tzinfo->StandardName, sizeof(tzinfo->StandardName) ))
+    {
+        NtClose( time_zone_key );
+        return FALSE;
+    }
+
+    if (!reg_load_mui_string( time_zone_key, mui_dltW, tzinfo->DaylightName, sizeof(tzinfo->DaylightName) ) &&
+        !reg_query_value( time_zone_key, dltW, REG_SZ, tzinfo->DaylightName, sizeof(tzinfo->DaylightName) ))
     {
         NtClose( time_zone_key );
         return FALSE;
@@ -577,17 +605,15 @@ BOOL WINAPI SetSystemTimeAdjustment( DWORD dwTimeAdjustment, BOOL bTimeAdjustmen
  *  TIME_ZONE_ID_STANDARD   Current time is standard time
  *  TIME_ZONE_ID_DAYLIGHT   Current time is daylight savings time
  */
-DWORD WINAPI GetTimeZoneInformation( LPTIME_ZONE_INFORMATION tzinfo )
+DWORD WINAPI GetTimeZoneInformation( LPTIME_ZONE_INFORMATION ret )
 {
-    NTSTATUS status;
+    DYNAMIC_TIME_ZONE_INFORMATION tzinfo;
+    DWORD time_zone_id;
 
-    status = RtlQueryTimeZoneInformation( (RTL_TIME_ZONE_INFORMATION*)tzinfo );
-    if ( status != STATUS_SUCCESS )
-    {
-        SetLastError( RtlNtStatusToDosError(status) );
-        return TIME_ZONE_ID_INVALID;
-    }
-    return TIME_ZoneID( tzinfo );
+    TRACE("(%p)\n", ret);
+    time_zone_id = GetDynamicTimeZoneInformation( &tzinfo );
+    memcpy( ret, &tzinfo, sizeof(*ret) );
+    return time_zone_id;
 }
 
 /***********************************************************************
@@ -598,6 +624,7 @@ BOOL WINAPI GetTimeZoneInformationForYear( USHORT wYear,
 {
     DYNAMIC_TIME_ZONE_INFORMATION local_dtzi, result;
 
+    TRACE("(%u,%p)\n", wYear, ptzi);
     if (!pdtzi)
     {
         if (GetDynamicTimeZoneInformation(&local_dtzi) == TIME_ZONE_ID_INVALID)
@@ -629,6 +656,8 @@ BOOL WINAPI GetTimeZoneInformationForYear( USHORT wYear,
 BOOL WINAPI SetTimeZoneInformation( const TIME_ZONE_INFORMATION *tzinfo )
 {
     NTSTATUS status;
+
+    TRACE("(%p)\n", tzinfo);
     status = RtlSetTimeZoneInformation( (const RTL_TIME_ZONE_INFORMATION *)tzinfo );
     if ( status != STATUS_SUCCESS )
         SetLastError( RtlNtStatusToDosError(status) );
@@ -665,18 +694,17 @@ BOOL WINAPI SystemTimeToTzSpecificLocalTime(
     }
     else
     {
-        if (GetTimeZoneInformation(&tzinfo) == TIME_ZONE_ID_INVALID)
-            return FALSE;
+        RtlQueryTimeZoneInformation((RTL_TIME_ZONE_INFORMATION *)&tzinfo);
     }
 
     if (!SystemTimeToFileTime(lpUniversalTime, &ft))
         return FALSE;
-    FILETIME2LL( &ft, llTime)
-    if (!TIME_GetTimezoneBias(&tzinfo, &ft, FALSE, &lBias))
+    llTime = filetime_to_longlong( &ft );
+    if (!TIME_GetTimezoneBias(&tzinfo, llTime, FALSE, &lBias))
         return FALSE;
     /* convert minutes to 100-nanoseconds-ticks */
     llTime -= (LONGLONG)lBias * 600000000;
-    LL2FILETIME( llTime, &ft)
+    longlong_to_filetime( llTime, &ft );
 
     return FileTimeToSystemTime(&ft, lpLocalTime);
 }
@@ -711,52 +739,39 @@ BOOL WINAPI TzSpecificLocalTimeToSystemTime(
     }
     else
     {
-        if (GetTimeZoneInformation(&tzinfo) == TIME_ZONE_ID_INVALID)
-            return FALSE;
+        RtlQueryTimeZoneInformation((RTL_TIME_ZONE_INFORMATION *)&tzinfo);
     }
 
     if (!SystemTimeToFileTime(lpLocalTime, &ft))
         return FALSE;
-    FILETIME2LL( &ft, t)
-    if (!TIME_GetTimezoneBias(&tzinfo, &ft, TRUE, &lBias))
+    t = filetime_to_longlong( &ft );
+    if (!TIME_GetTimezoneBias(&tzinfo, t, TRUE, &lBias))
         return FALSE;
     /* convert minutes to 100-nanoseconds-ticks */
     t += (LONGLONG)lBias * 600000000;
-    LL2FILETIME( t, &ft)
+    longlong_to_filetime( t, &ft );
     return FileTimeToSystemTime(&ft, lpUniversalTime);
 }
 
 
-/***********************************************************************
- *              GetSystemTimeAsFileTime  (KERNEL32.@)
- *
- *  Get the current time in utc format.
- *
- *  RETURNS
- *   Nothing.
- */
-VOID WINAPI GetSystemTimeAsFileTime(
-    LPFILETIME time) /* [out] Destination for the current utc time */
-{
-    LARGE_INTEGER t;
-    NtQuerySystemTime( &t );
-    time->dwLowDateTime = t.u.LowPart;
-    time->dwHighDateTime = t.u.HighPart;
-}
-
-
 /***********************************************************************
  *              GetSystemTimePreciseAsFileTime  (KERNEL32.@)
  *
- *  Get the current time in utc format, with <1 us precision.
+ *  Get the current time in utc format with greater accuracy.
+ *
+ *  PARAMS
+ *   time [out] Destination for the current utc time
  *
  *  RETURNS
  *   Nothing.
  */
-VOID WINAPI GetSystemTimePreciseAsFileTime(
-    LPFILETIME time) /* [out] Destination for the current utc time */
+void WINAPI GetSystemTimePreciseAsFileTime( FILETIME *time )
 {
-    GetSystemTimeAsFileTime(time);
+    LARGE_INTEGER t;
+
+    t.QuadPart = RtlGetSystemTimePrecise();
+    time->dwLowDateTime = t.u.LowPart;
+    time->dwHighDateTime = t.u.HighPart;
 }
 
 
@@ -778,6 +793,46 @@ static void TIME_ClockTimeToFileTime(clock_t unix_time, LPFILETIME filetime)
     filetime->dwHighDateTime = (DWORD)(secs >> 32);
 }
 
+/***********************************************************************
+ *		TIMEZONE_InitRegistry
+ *
+ * Update registry contents on startup if the user timezone has changed.
+ * This simulates the action of the Windows control panel.
+ */
+void TIMEZONE_InitRegistry(void)
+{
+    static const WCHAR timezoneInformationW[] = {
+        'M','a','c','h','i','n','e','\\','S','y','s','t','e','m','\\',
+        'C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t','\\',
+        'C','o','n','t','r','o','l','\\',
+        'T','i','m','e','Z','o','n','e','I','n','f','o','r','m','a','t','i','o','n','\0'
+    };
+    static const WCHAR standardNameW[] = {'S','t','a','n','d','a','r','d','N','a','m','e','\0'};
+    static const WCHAR timezoneKeyNameW[] = {'T','i','m','e','Z','o','n','e','K','e','y','N','a','m','e','\0'};
+    DYNAMIC_TIME_ZONE_INFORMATION tzinfo;
+    UNICODE_STRING name;
+    OBJECT_ATTRIBUTES attr;
+    HANDLE hkey;
+    DWORD tzid;
+
+    tzid = GetDynamicTimeZoneInformation(&tzinfo);
+    if (tzid == TIME_ZONE_ID_INVALID) return;
+
+    RtlInitUnicodeString(&name, timezoneInformationW);
+    InitializeObjectAttributes(&attr, &name, 0, 0, NULL);
+    if (NtCreateKey(&hkey, KEY_ALL_ACCESS, &attr, 0, NULL, 0, NULL) != STATUS_SUCCESS) return;
+
+    RtlInitUnicodeString(&name, standardNameW);
+    NtSetValueKey(hkey, &name, 0, REG_SZ, tzinfo.StandardName,
+                  (strlenW(tzinfo.StandardName) + 1) * sizeof(WCHAR));
+
+    RtlInitUnicodeString(&name, timezoneKeyNameW);
+    NtSetValueKey(hkey, &name, 0, REG_SZ, tzinfo.TimeZoneKeyName,
+                  (strlenW(tzinfo.TimeZoneKeyName) + 1) * sizeof(WCHAR));
+
+    NtClose( hkey );
+}
+
 /*********************************************************************
  *	GetProcessTimes				(KERNEL32.@)
  *
@@ -813,8 +868,8 @@ BOOL WINAPI GetProcessTimes( HANDLE hprocess, LPFILETIME lpCreationTime,
     TIME_ClockTimeToFileTime(tms.tms_stime,lpKernelTime);
     if (NtQueryInformationProcess( hprocess, ProcessTimes, &pti, sizeof(pti), NULL))
         return FALSE;
-    LL2FILETIME( pti.CreateTime.QuadPart, lpCreationTime);
-    LL2FILETIME( pti.ExitTime.QuadPart, lpExitTime);
+    longlong_to_filetime( pti.CreateTime.QuadPart, lpCreationTime );
+    longlong_to_filetime( pti.ExitTime.QuadPart, lpExitTime );
     return TRUE;
 }
 
@@ -1005,7 +1060,15 @@ int WINAPI GetCalendarInfoW(LCID Locale, CALID Calendar, CALTYPE CalType,
 	case CAL_SABBREVMONTHNAME11:
 	case CAL_SABBREVMONTHNAME12:
 	case CAL_SABBREVMONTHNAME13:
+	case CAL_SMONTHDAY:
 	case CAL_SYEARMONTH:
+	case CAL_SSHORTESTDAYNAME1:
+	case CAL_SSHORTESTDAYNAME2:
+	case CAL_SSHORTESTDAYNAME3:
+	case CAL_SSHORTESTDAYNAME4:
+	case CAL_SSHORTESTDAYNAME5:
+	case CAL_SSHORTESTDAYNAME6:
+	case CAL_SSHORTESTDAYNAME7:
             return GetLocaleInfoW(Locale, caltype_lctype_map[calinfo] | localeflags, lpCalData, cchData);
 	case CAL_ITWODIGITYEARMAX:
             if (CalType & CAL_RETURN_NUMBER)
@@ -1028,6 +1091,9 @@ int WINAPI GetCalendarInfoW(LCID Locale, CALID Calendar, CALTYPE CalType,
                 return 0;
             }
 	    break;
+	case CAL_SABBREVERASTRING:
+            FIXME("Unimplemented caltype %d\n", calinfo);
+	    return 0;
 	default:
             FIXME("Unknown caltype %d\n", calinfo);
             SetLastError(ERROR_INVALID_FLAGS);
@@ -1250,7 +1316,10 @@ VOID WINAPI GetSystemTime(LPSYSTEMTIME systime)
 BOOL WINAPI GetDaylightFlag(void)
 {
     TIME_ZONE_INFORMATION tzinfo;
-    return GetTimeZoneInformation( &tzinfo) == TIME_ZONE_ID_DAYLIGHT;
+
+    TRACE("()\n");
+    RtlQueryTimeZoneInformation((RTL_TIME_ZONE_INFORMATION *)&tzinfo);
+    return (TIME_ZoneID(&tzinfo) == TIME_ZONE_ID_DAYLIGHT);
 }
 
 /***********************************************************************
@@ -1397,16 +1466,34 @@ BOOL WINAPI GetSystemTimes(LPFILETIME lpIdleTime, LPFILETIME lpKernelTime, LPFIL
 DWORD WINAPI GetDynamicTimeZoneInformation(DYNAMIC_TIME_ZONE_INFORMATION *tzinfo)
 {
     NTSTATUS status;
+    HANDLE time_zone_key;
 
+    TRACE("(%p)\n", tzinfo);
     status = RtlQueryDynamicTimeZoneInformation( (RTL_DYNAMIC_TIME_ZONE_INFORMATION*)tzinfo );
     if ( status != STATUS_SUCCESS )
     {
         SetLastError( RtlNtStatusToDosError(status) );
         return TIME_ZONE_ID_INVALID;
     }
+
+    if (!TIME_GetSpecificTimeZoneKey( tzinfo->TimeZoneKeyName, &time_zone_key ))
+        return TIME_ZONE_ID_INVALID;
+    reg_load_mui_string( time_zone_key, mui_stdW, tzinfo->StandardName, sizeof(tzinfo->StandardName) );
+    reg_load_mui_string( time_zone_key, mui_dltW, tzinfo->DaylightName, sizeof(tzinfo->DaylightName) );
+    NtClose( time_zone_key );
+
     return TIME_ZoneID( (TIME_ZONE_INFORMATION*)tzinfo );
 }
 
+/***********************************************************************
+ *           GetDynamicTimeZoneInformationEffectiveYears   (KERNEL32.@)
+ */
+DWORD WINAPI GetDynamicTimeZoneInformationEffectiveYears(DYNAMIC_TIME_ZONE_INFORMATION *tzinfo, DWORD *first_year, DWORD *last_year)
+{
+    FIXME("(%p, %p, %p): stub!\n", tzinfo, first_year, last_year);
+    return ERROR_FILE_NOT_FOUND;
+}
+
 /***********************************************************************
  *           QueryProcessCycleTime   (KERNEL32.@)
  */
