@@ -33,11 +33,36 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(mfplat);
 
+enum session_command
+{
+    SESSION_CLEAR_TOPOLOGIES,
+};
+
+struct session_op
+{
+    IUnknown IUnknown_iface;
+    LONG refcount;
+    enum session_command command;
+};
+
+struct queued_topology
+{
+    struct list entry;
+    IMFTopology *topology;
+};
+
 struct media_session
 {
     IMFMediaSession IMFMediaSession_iface;
+    IMFGetService IMFGetService_iface;
+    IMFRateSupport IMFRateSupport_iface;
+    IMFRateControl IMFRateControl_iface;
+    IMFAsyncCallback commands_callback;
     LONG refcount;
     IMFMediaEventQueue *event_queue;
+    IMFPresentationClock *clock;
+    struct list topologies;
+    CRITICAL_SECTION cs;
 };
 
 struct clock_sink
@@ -92,6 +117,31 @@ static inline struct media_session *impl_from_IMFMediaSession(IMFMediaSession *i
     return CONTAINING_RECORD(iface, struct media_session, IMFMediaSession_iface);
 }
 
+static struct media_session *impl_from_commands_callback_IMFAsyncCallback(IMFAsyncCallback *iface)
+{
+    return CONTAINING_RECORD(iface, struct media_session, commands_callback);
+}
+
+static struct media_session *impl_from_IMFGetService(IMFGetService *iface)
+{
+    return CONTAINING_RECORD(iface, struct media_session, IMFGetService_iface);
+}
+
+static struct media_session *impl_session_from_IMFRateSupport(IMFRateSupport *iface)
+{
+    return CONTAINING_RECORD(iface, struct media_session, IMFRateSupport_iface);
+}
+
+static struct media_session *impl_session_from_IMFRateControl(IMFRateControl *iface)
+{
+    return CONTAINING_RECORD(iface, struct media_session, IMFRateControl_iface);
+}
+
+static struct session_op *impl_op_from_IUnknown(IUnknown *iface)
+{
+    return CONTAINING_RECORD(iface, struct session_op, IUnknown_iface);
+}
+
 static struct presentation_clock *impl_from_IMFPresentationClock(IMFPresentationClock *iface)
 {
     return CONTAINING_RECORD(iface, struct presentation_clock, IMFPresentationClock_iface);
@@ -122,6 +172,79 @@ static struct sink_notification *impl_from_IUnknown(IUnknown *iface)
     return CONTAINING_RECORD(iface, struct sink_notification, IUnknown_iface);
 }
 
+static HRESULT WINAPI session_op_QueryInterface(IUnknown *iface, REFIID riid, void **obj)
+{
+    if (IsEqualIID(riid, &IID_IUnknown))
+    {
+        *obj = iface;
+        IUnknown_AddRef(iface);
+        return S_OK;
+    }
+
+    *obj = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI session_op_AddRef(IUnknown *iface)
+{
+    struct session_op *op = impl_op_from_IUnknown(iface);
+    ULONG refcount = InterlockedIncrement(&op->refcount);
+
+    TRACE("%p, refcount %u.\n", iface, refcount);
+
+    return refcount;
+}
+
+static ULONG WINAPI session_op_Release(IUnknown *iface)
+{
+    struct session_op *op = impl_op_from_IUnknown(iface);
+    ULONG refcount = InterlockedDecrement(&op->refcount);
+
+    TRACE("%p, refcount %u.\n", iface, refcount);
+
+    if (!refcount)
+    {
+        heap_free(op);
+    }
+
+    return refcount;
+}
+
+static IUnknownVtbl session_op_vtbl =
+{
+    session_op_QueryInterface,
+    session_op_AddRef,
+    session_op_Release,
+};
+
+static HRESULT create_session_op(enum session_command command, IUnknown **ret)
+{
+    struct session_op *op;
+
+    if (!(op = heap_alloc(sizeof(*op))))
+        return E_OUTOFMEMORY;
+
+    op->IUnknown_iface.lpVtbl = &session_op_vtbl;
+    op->refcount = 1;
+    op->command = command;
+
+    *ret = &op->IUnknown_iface;
+
+    return S_OK;
+}
+
+static void session_clear_topologies(struct media_session *session)
+{
+    struct queued_topology *ptr, *next;
+
+    LIST_FOR_EACH_ENTRY_SAFE(ptr, next, &session->topologies, struct queued_topology, entry)
+    {
+        list_remove(&ptr->entry);
+        IMFTopology_Release(ptr->topology);
+        heap_free(ptr);
+    }
+}
+
 static HRESULT WINAPI mfsession_QueryInterface(IMFMediaSession *iface, REFIID riid, void **out)
 {
     struct media_session *session = impl_from_IMFMediaSession(iface);
@@ -136,6 +259,12 @@ static HRESULT WINAPI mfsession_QueryInterface(IMFMediaSession *iface, REFIID ri
         IMFMediaSession_AddRef(iface);
         return S_OK;
     }
+    else if (IsEqualIID(riid, &IID_IMFGetService))
+    {
+        *out = &session->IMFGetService_iface;
+        IMFMediaSession_AddRef(iface);
+        return S_OK;
+    }
 
     WARN("Unsupported %s.\n", debugstr_guid(riid));
     *out = NULL;
@@ -161,8 +290,12 @@ static ULONG WINAPI mfsession_Release(IMFMediaSession *iface)
 
     if (!refcount)
     {
+        session_clear_topologies(session);
         if (session->event_queue)
             IMFMediaEventQueue_Release(session->event_queue);
+        if (session->clock)
+            IMFPresentationClock_Release(session->clock);
+        DeleteCriticalSection(&session->cs);
         heap_free(session);
     }
 
@@ -208,16 +341,44 @@ static HRESULT WINAPI mfsession_QueueEvent(IMFMediaSession *iface, MediaEventTyp
 
 static HRESULT WINAPI mfsession_SetTopology(IMFMediaSession *iface, DWORD flags, IMFTopology *topology)
 {
-    FIXME("(%p)->(%#x, %p)\n", iface, flags, topology);
+    struct media_session *session = impl_from_IMFMediaSession(iface);
+    struct queued_topology *queued_topology;
 
-    return E_NOTIMPL;
+    FIXME("%p, %#x, %p.\n", iface, flags, topology);
+
+    if (!(queued_topology = heap_alloc(sizeof(*queued_topology))))
+        return E_OUTOFMEMORY;
+
+    queued_topology->topology = topology;
+    IMFTopology_AddRef(queued_topology->topology);
+
+    EnterCriticalSection(&session->cs);
+    list_add_tail(&session->topologies, &queued_topology->entry);
+    LeaveCriticalSection(&session->cs);
+
+    return S_OK;
+}
+
+static HRESULT session_submit_command(struct media_session *session, IUnknown *op)
+{
+    return MFPutWorkItem(MFASYNC_CALLBACK_QUEUE_STANDARD, &session->commands_callback, op);
 }
 
 static HRESULT WINAPI mfsession_ClearTopologies(IMFMediaSession *iface)
 {
-    FIXME("(%p)\n", iface);
+    struct media_session *session = impl_from_IMFMediaSession(iface);
+    IUnknown *op;
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("%p.\n", iface);
+
+    if (FAILED(hr = create_session_op(SESSION_CLEAR_TOPOLOGIES, &op)))
+        return hr;
+
+    hr = session_submit_command(session, op);
+    IUnknown_Release(op);
+
+    return hr;
 }
 
 static HRESULT WINAPI mfsession_Start(IMFMediaSession *iface, const GUID *format, const PROPVARIANT *start)
@@ -261,9 +422,14 @@ static HRESULT WINAPI mfsession_Shutdown(IMFMediaSession *iface)
 
 static HRESULT WINAPI mfsession_GetClock(IMFMediaSession *iface, IMFClock **clock)
 {
-    FIXME("(%p)->(%p)\n", iface, clock);
+    struct media_session *session = impl_from_IMFMediaSession(iface);
 
-    return E_NOTIMPL;
+    TRACE("%p, %p.\n", iface, clock);
+
+    *clock = (IMFClock *)session->clock;
+    IMFClock_AddRef(*clock);
+
+    return S_OK;
 }
 
 static HRESULT WINAPI mfsession_GetSessionCapabilities(IMFMediaSession *iface, DWORD *caps)
@@ -301,6 +467,218 @@ static const IMFMediaSessionVtbl mfmediasessionvtbl =
     mfsession_GetFullTopology,
 };
 
+static HRESULT WINAPI session_get_service_QueryInterface(IMFGetService *iface, REFIID riid, void **obj)
+{
+    struct media_session *session = impl_from_IMFGetService(iface);
+    return IMFMediaSession_QueryInterface(&session->IMFMediaSession_iface, riid, obj);
+}
+
+static ULONG WINAPI session_get_service_AddRef(IMFGetService *iface)
+{
+    struct media_session *session = impl_from_IMFGetService(iface);
+    return IMFMediaSession_AddRef(&session->IMFMediaSession_iface);
+}
+
+static ULONG WINAPI session_get_service_Release(IMFGetService *iface)
+{
+    struct media_session *session = impl_from_IMFGetService(iface);
+    return IMFMediaSession_Release(&session->IMFMediaSession_iface);
+}
+
+static HRESULT WINAPI session_get_service_GetService(IMFGetService *iface, REFGUID service, REFIID riid, void **obj)
+{
+    struct media_session *session = impl_from_IMFGetService(iface);
+
+    TRACE("%p, %s, %s, %p.\n", iface, debugstr_guid(service), debugstr_guid(riid), obj);
+
+    *obj = NULL;
+
+    if (IsEqualGUID(service, &MF_RATE_CONTROL_SERVICE))
+    {
+        if (IsEqualIID(riid, &IID_IMFRateSupport))
+        {
+            *obj = &session->IMFRateSupport_iface;
+        }
+        else if (IsEqualIID(riid, &IID_IMFRateControl))
+        {
+            *obj = &session->IMFRateControl_iface;
+        }
+
+        if (*obj)
+            IUnknown_AddRef((IUnknown *)*obj);
+
+        return *obj ? S_OK : E_NOINTERFACE;
+    }
+    else
+        FIXME("Unsupported service %s.\n", debugstr_guid(service));
+
+    return E_NOTIMPL;
+}
+
+static const IMFGetServiceVtbl session_get_service_vtbl =
+{
+    session_get_service_QueryInterface,
+    session_get_service_AddRef,
+    session_get_service_Release,
+    session_get_service_GetService,
+};
+
+static HRESULT WINAPI session_commands_callback_QueryInterface(IMFAsyncCallback *iface, REFIID riid, void **obj)
+{
+    if (IsEqualIID(riid, &IID_IMFAsyncCallback) ||
+            IsEqualIID(riid, &IID_IUnknown))
+    {
+        *obj = iface;
+        IMFAsyncCallback_AddRef(iface);
+        return S_OK;
+    }
+
+    WARN("Unsupported %s.\n", debugstr_guid(riid));
+    *obj = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI session_commands_callback_AddRef(IMFAsyncCallback *iface)
+{
+    struct media_session *session = impl_from_commands_callback_IMFAsyncCallback(iface);
+    return IMFMediaSession_AddRef(&session->IMFMediaSession_iface);
+}
+
+static ULONG WINAPI session_commands_callback_Release(IMFAsyncCallback *iface)
+{
+    struct media_session *session = impl_from_commands_callback_IMFAsyncCallback(iface);
+    return IMFMediaSession_Release(&session->IMFMediaSession_iface);
+}
+
+static HRESULT WINAPI session_commands_callback_GetParameters(IMFAsyncCallback *iface, DWORD *flags, DWORD *queue)
+{
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI session_commands_callback_Invoke(IMFAsyncCallback *iface, IMFAsyncResult *result)
+{
+    struct session_op *op = impl_op_from_IUnknown(IMFAsyncResult_GetStateNoAddRef(result));
+    struct media_session *session = impl_from_commands_callback_IMFAsyncCallback(iface);
+
+    switch (op->command)
+    {
+        case SESSION_CLEAR_TOPOLOGIES:
+            EnterCriticalSection(&session->cs);
+            session_clear_topologies(session);
+            LeaveCriticalSection(&session->cs);
+
+            IMFMediaEventQueue_QueueEventParamVar(session->event_queue, MESessionTopologiesCleared, &GUID_NULL,
+                    S_OK, NULL);
+            break;
+        default:
+            ;
+    }
+
+    return S_OK;
+}
+
+static const IMFAsyncCallbackVtbl session_commands_callback_vtbl =
+{
+    session_commands_callback_QueryInterface,
+    session_commands_callback_AddRef,
+    session_commands_callback_Release,
+    session_commands_callback_GetParameters,
+    session_commands_callback_Invoke,
+};
+
+static HRESULT WINAPI session_rate_support_QueryInterface(IMFRateSupport *iface, REFIID riid, void **obj)
+{
+    struct media_session *session = impl_session_from_IMFRateSupport(iface);
+    return IMFMediaSession_QueryInterface(&session->IMFMediaSession_iface, riid, obj);
+}
+
+static ULONG WINAPI session_rate_support_AddRef(IMFRateSupport *iface)
+{
+    struct media_session *session = impl_session_from_IMFRateSupport(iface);
+    return IMFMediaSession_AddRef(&session->IMFMediaSession_iface);
+}
+
+static ULONG WINAPI session_rate_support_Release(IMFRateSupport *iface)
+{
+    struct media_session *session = impl_session_from_IMFRateSupport(iface);
+    return IMFMediaSession_Release(&session->IMFMediaSession_iface);
+}
+
+static HRESULT WINAPI session_rate_support_GetSlowestRate(IMFRateSupport *iface, MFRATE_DIRECTION direction,
+        BOOL thin, float *rate)
+{
+    FIXME("%p, %d, %d, %p.\n", iface, direction, thin, rate);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI session_rate_support_GetFastestRate(IMFRateSupport *iface, MFRATE_DIRECTION direction,
+        BOOL thin, float *rate)
+{
+    FIXME("%p, %d, %d, %p.\n", iface, direction, thin, rate);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI session_rate_support_IsSupported(IMFRateSupport *iface, BOOL thin, float rate,
+        float *nearest_supported_rate)
+{
+    FIXME("%p, %d, %f, %p.\n", iface, thin, rate, nearest_supported_rate);
+
+    return E_NOTIMPL;
+}
+
+static const IMFRateSupportVtbl session_rate_support_vtbl =
+{
+    session_rate_support_QueryInterface,
+    session_rate_support_AddRef,
+    session_rate_support_Release,
+    session_rate_support_GetSlowestRate,
+    session_rate_support_GetFastestRate,
+    session_rate_support_IsSupported,
+};
+
+static HRESULT WINAPI session_rate_control_QueryInterface(IMFRateControl *iface, REFIID riid, void **obj)
+{
+    struct media_session *session = impl_session_from_IMFRateControl(iface);
+    return IMFMediaSession_QueryInterface(&session->IMFMediaSession_iface, riid, obj);
+}
+
+static ULONG WINAPI session_rate_control_AddRef(IMFRateControl *iface)
+{
+    struct media_session *session = impl_session_from_IMFRateControl(iface);
+    return IMFMediaSession_AddRef(&session->IMFMediaSession_iface);
+}
+
+static ULONG WINAPI session_rate_control_Release(IMFRateControl *iface)
+{
+    struct media_session *session = impl_session_from_IMFRateControl(iface);
+    return IMFMediaSession_Release(&session->IMFMediaSession_iface);
+}
+
+static HRESULT WINAPI session_rate_control_SetRate(IMFRateControl *iface, BOOL thin, float rate)
+{
+    FIXME("%p, %d, %f.\n", iface, thin, rate);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI session_rate_control_GetRate(IMFRateControl *iface, BOOL *thin, float *rate)
+{
+    FIXME("%p, %p, %p.\n", iface, thin, rate);
+
+    return E_NOTIMPL;
+}
+
+static const IMFRateControlVtbl session_rate_control_vtbl =
+{
+    session_rate_control_QueryInterface,
+    session_rate_control_AddRef,
+    session_rate_control_Release,
+    session_rate_control_SetRate,
+    session_rate_control_GetRate,
+};
+
 /***********************************************************************
  *      MFCreateMediaSession (mf.@)
  */
@@ -319,16 +697,27 @@ HRESULT WINAPI MFCreateMediaSession(IMFAttributes *config, IMFMediaSession **ses
         return E_OUTOFMEMORY;
 
     object->IMFMediaSession_iface.lpVtbl = &mfmediasessionvtbl;
+    object->IMFGetService_iface.lpVtbl = &session_get_service_vtbl;
+    object->IMFRateSupport_iface.lpVtbl = &session_rate_support_vtbl;
+    object->IMFRateControl_iface.lpVtbl = &session_rate_control_vtbl;
+    object->commands_callback.lpVtbl = &session_commands_callback_vtbl;
     object->refcount = 1;
+    list_init(&object->topologies);
+    InitializeCriticalSection(&object->cs);
+
     if (FAILED(hr = MFCreateEventQueue(&object->event_queue)))
-    {
-        IMFMediaSession_Release(&object->IMFMediaSession_iface);
-        return hr;
-    }
+        goto failed;
+
+    if (FAILED(hr = MFCreatePresentationClock(&object->clock)))
+        goto failed;
 
     *session = &object->IMFMediaSession_iface;
 
     return S_OK;
+
+failed:
+    IMFMediaSession_Release(&object->IMFMediaSession_iface);
+    return hr;
 }
 
 static HRESULT WINAPI present_clock_QueryInterface(IMFPresentationClock *iface, REFIID riid, void **out)
@@ -442,9 +831,24 @@ static HRESULT WINAPI present_clock_GetState(IMFPresentationClock *iface, DWORD
 
 static HRESULT WINAPI present_clock_GetProperties(IMFPresentationClock *iface, MFCLOCK_PROPERTIES *props)
 {
-    FIXME("%p, %p.\n", iface, props);
+    struct presentation_clock *clock = impl_from_IMFPresentationClock(iface);
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("%p, %p.\n", iface, props);
+
+    EnterCriticalSection(&clock->cs);
+
+    if (clock->time_source)
+    {
+        FIXME("%p, %p.\n", iface, props);
+        hr = E_NOTIMPL;
+    }
+    else
+        hr = MF_E_CLOCK_NO_TIME_SOURCE;
+
+    LeaveCriticalSection(&clock->cs);
+
+    return hr;
 }
 
 static HRESULT WINAPI present_clock_SetTimeSource(IMFPresentationClock *iface,
