@@ -104,7 +104,7 @@ static ULONG WINAPI Unknown_AddRef(IUnknown *iface)
 static ULONG WINAPI Unknown_Release(IUnknown *iface)
 {
     SmartTeeFilter *This = impl_from_IUnknown(iface);
-    ULONG ref = BaseFilterImpl_Release(&This->filter.IBaseFilter_iface);
+    ULONG ref = InterlockedDecrement(&This->filter.refCount);
 
     TRACE("(%p)->() ref=%d\n", This, ref);
 
@@ -115,6 +115,7 @@ static ULONG WINAPI Unknown_Release(IUnknown *iface)
             BaseOutputPinImpl_Release(&This->capture->pin.IPin_iface);
         if(This->preview)
             BaseOutputPinImpl_Release(&This->preview->pin.IPin_iface);
+        strmbase_filter_cleanup(&This->filter);
         CoTaskMemFree(This);
     }
     return ref;
@@ -180,26 +181,6 @@ static HRESULT WINAPI SmartTeeFilter_Run(IBaseFilter *iface, REFERENCE_TIME tSta
     return hr;
 }
 
-static HRESULT WINAPI SmartTeeFilter_FindPin(IBaseFilter *iface, LPCWSTR Id, IPin **ppPin)
-{
-    SmartTeeFilter *This = impl_from_IBaseFilter(iface);
-    TRACE("(%p)->(%s, %p)\n", This, debugstr_w(Id), ppPin);
-    if (lstrcmpW(Id, This->input->pin.pinInfo.achName) == 0) {
-        *ppPin = &This->input->pin.IPin_iface;
-        IPin_AddRef(*ppPin);
-        return S_OK;
-    } else if (lstrcmpW(Id, This->capture->pin.pinInfo.achName) == 0) {
-        *ppPin = &This->capture->pin.IPin_iface;
-        IPin_AddRef(*ppPin);
-        return S_OK;
-    } else if (lstrcmpW(Id, This->preview->pin.pinInfo.achName) == 0) {
-        *ppPin = &This->preview->pin.IPin_iface;
-        IPin_AddRef(*ppPin);
-        return S_OK;
-    }
-    return VFW_E_NOT_FOUND;
-}
-
 static const IBaseFilterVtbl SmartTeeFilterVtbl = {
     SmartTeeFilter_QueryInterface,
     SmartTeeFilter_AddRef,
@@ -212,45 +193,32 @@ static const IBaseFilterVtbl SmartTeeFilterVtbl = {
     BaseFilterImpl_SetSyncSource,
     BaseFilterImpl_GetSyncSource,
     BaseFilterImpl_EnumPins,
-    SmartTeeFilter_FindPin,
+    BaseFilterImpl_FindPin,
     BaseFilterImpl_QueryFilterInfo,
     BaseFilterImpl_JoinFilterGraph,
     BaseFilterImpl_QueryVendorInfo
 };
 
-static IPin* WINAPI SmartTeeFilter_GetPin(BaseFilter *iface, int pos)
+static IPin *smart_tee_get_pin(BaseFilter *iface, unsigned int index)
 {
     SmartTeeFilter *This = impl_from_BaseFilter(iface);
     IPin *ret;
 
-    TRACE("(%p)->(%d)\n", This, pos);
-
-    switch(pos) {
-    case 0:
+    if (index == 0)
         ret = &This->input->pin.IPin_iface;
-        break;
-    case 1:
+    else if (index == 1)
         ret = &This->capture->pin.IPin_iface;
-        break;
-    case 2:
+    else if (index == 2)
         ret = &This->preview->pin.IPin_iface;
-        break;
-    default:
-        TRACE("No pin %d\n", pos);
+    else
         return NULL;
-    }
 
     IPin_AddRef(ret);
     return ret;
 }
 
-static LONG WINAPI SmartTeeFilter_GetPinCount(BaseFilter *iface)
-{
-    return 3;
-}
 static const BaseFilterFuncTable SmartTeeFilterFuncs = {
-    SmartTeeFilter_GetPin,
-    SmartTeeFilter_GetPinCount
+    .filter_get_pin = smart_tee_get_pin,
 };
 
 static ULONG WINAPI SmartTeeFilterInput_AddRef(IPin *iface)
@@ -299,11 +267,6 @@ static HRESULT WINAPI SmartTeeFilterInput_CheckMediaType(BasePin *base, const AM
     return S_OK;
 }
 
-static LONG WINAPI SmartTeeFilterInput_GetMediaTypeVersion(BasePin *base)
-{
-    return 0;
-}
-
 static HRESULT WINAPI SmartTeeFilterInput_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
 {
     SmartTeeFilter *This = impl_from_BasePin(base);
@@ -445,8 +408,6 @@ static HRESULT WINAPI SmartTeeFilterInput_Receive(BaseInputPin *base, IMediaSamp
 static const BaseInputPinFuncTable SmartTeeFilterInputFuncs = {
     {
         SmartTeeFilterInput_CheckMediaType,
-        NULL,
-        SmartTeeFilterInput_GetMediaTypeVersion,
         SmartTeeFilterInput_GetMediaType
     },
     SmartTeeFilterInput_Receive
@@ -505,13 +466,6 @@ static HRESULT WINAPI SmartTeeFilterCapture_CheckMediaType(BasePin *base, const
     return S_OK;
 }
 
-static LONG WINAPI SmartTeeFilterCapture_GetMediaTypeVersion(BasePin *base)
-{
-    SmartTeeFilter *This = impl_from_BasePin(base);
-    TRACE("(%p)\n", This);
-    return 0;
-}
-
 static HRESULT WINAPI SmartTeeFilterCapture_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
 {
     SmartTeeFilter *This = impl_from_BasePin(base);
@@ -532,23 +486,14 @@ static HRESULT WINAPI SmartTeeFilterCapture_DecideAllocator(BaseOutputPin *base,
     return IMemInputPin_NotifyAllocator(pPin, This->input->pAllocator, TRUE);
 }
 
-static HRESULT WINAPI SmartTeeFilterCapture_BreakConnect(BaseOutputPin *base)
-{
-    SmartTeeFilter *This = impl_from_BasePin(&base->pin);
-    FIXME("(%p): stub\n", This);
-    return E_NOTIMPL;
-}
-
 static const BaseOutputPinFuncTable SmartTeeFilterCaptureFuncs = {
     {
         SmartTeeFilterCapture_CheckMediaType,
-        BaseOutputPinImpl_AttemptConnection,
-        SmartTeeFilterCapture_GetMediaTypeVersion,
         SmartTeeFilterCapture_GetMediaType
     },
+    BaseOutputPinImpl_AttemptConnection,
     NULL,
     SmartTeeFilterCapture_DecideAllocator,
-    SmartTeeFilterCapture_BreakConnect
 };
 
 static ULONG WINAPI SmartTeeFilterPreview_AddRef(IPin *iface)
@@ -604,13 +549,6 @@ static HRESULT WINAPI SmartTeeFilterPreview_CheckMediaType(BasePin *base, const
     return S_OK;
 }
 
-static LONG WINAPI SmartTeeFilterPreview_GetMediaTypeVersion(BasePin *base)
-{
-    SmartTeeFilter *This = impl_from_BasePin(base);
-    TRACE("(%p)\n", This);
-    return 0;
-}
-
 static HRESULT WINAPI SmartTeeFilterPreview_GetMediaType(BasePin *base, int iPosition, AM_MEDIA_TYPE *amt)
 {
     SmartTeeFilter *This = impl_from_BasePin(base);
@@ -631,23 +569,14 @@ static HRESULT WINAPI SmartTeeFilterPreview_DecideAllocator(BaseOutputPin *base,
     return IMemInputPin_NotifyAllocator(pPin, This->input->pAllocator, TRUE);
 }
 
-static HRESULT WINAPI SmartTeeFilterPreview_BreakConnect(BaseOutputPin *base)
-{
-    SmartTeeFilter *This = impl_from_BasePin(&base->pin);
-    FIXME("(%p): stub\n", This);
-    return E_NOTIMPL;
-}
-
 static const BaseOutputPinFuncTable SmartTeeFilterPreviewFuncs = {
     {
         SmartTeeFilterPreview_CheckMediaType,
-        BaseOutputPinImpl_AttemptConnection,
-        SmartTeeFilterPreview_GetMediaTypeVersion,
         SmartTeeFilterPreview_GetMediaType
     },
+    BaseOutputPinImpl_AttemptConnection,
     NULL,
     SmartTeeFilterPreview_DecideAllocator,
-    SmartTeeFilterPreview_BreakConnect
 };
 IUnknown* WINAPI QCAP_createSmartTeeFilter(IUnknown *outer, HRESULT *phr)
 {
@@ -661,8 +590,8 @@ IUnknown* WINAPI QCAP_createSmartTeeFilter(IUnknown *outer, HRESULT *phr)
 
     This = CoTaskMemAlloc(sizeof(*This));
     if (This == NULL) {
-        hr = E_OUTOFMEMORY;
-        goto end;
+        *phr = E_OUTOFMEMORY;
+        return NULL;
     }
     memset(This, 0, sizeof(*This));
     This->IUnknown_iface.lpVtbl = &UnknownVtbl;
@@ -702,8 +631,7 @@ end:
         else
             return (IUnknown*)&This->filter.IBaseFilter_iface;
     } else {
-        if (This)
-            IBaseFilter_Release(&This->filter.IBaseFilter_iface);
+        strmbase_filter_cleanup(&This->filter);
         return NULL;
     }
 }
