@@ -50,8 +50,6 @@ typedef struct AVIDecImpl
     REFERENCE_TIME late;
 } AVIDecImpl;
 
-static const IBaseFilterVtbl AVIDec_Vtbl;
-
 static inline AVIDecImpl *impl_from_TransformFilter( TransformFilter *iface )
 {
     return CONTAINING_RECORD(iface, AVIDecImpl, tf);
@@ -404,7 +402,8 @@ HRESULT AVIDec_create(IUnknown * pUnkOuter, LPVOID * ppv)
     if (pUnkOuter)
         return CLASS_E_NOAGGREGATION;
 
-    hr = TransformFilter_Construct(&AVIDec_Vtbl, sizeof(AVIDecImpl), &CLSID_AVIDec, &AVIDec_FuncsTable, (IBaseFilter**)&This);
+    hr = strmbase_transform_create(sizeof(AVIDecImpl), &CLSID_AVIDec,
+            &AVIDec_FuncsTable, (IBaseFilter **)&This);
 
     if (FAILED(hr))
         return hr;
@@ -417,22 +416,3 @@ HRESULT AVIDec_create(IUnknown * pUnkOuter, LPVOID * ppv)
 
     return hr;
 }
-
-static const IBaseFilterVtbl AVIDec_Vtbl =
-{
-    TransformFilterImpl_QueryInterface,
-    BaseFilterImpl_AddRef,
-    TransformFilterImpl_Release,
-    BaseFilterImpl_GetClassID,
-    TransformFilterImpl_Stop,
-    TransformFilterImpl_Pause,
-    TransformFilterImpl_Run,
-    BaseFilterImpl_GetState,
-    BaseFilterImpl_SetSyncSource,
-    BaseFilterImpl_GetSyncSource,
-    BaseFilterImpl_EnumPins,
-    BaseFilterImpl_FindPin,
-    BaseFilterImpl_QueryFilterInfo,
-    BaseFilterImpl_JoinFilterGraph,
-    BaseFilterImpl_QueryVendorInfo
-};
