@@ -105,6 +105,11 @@ static inline AVISplitterImpl *impl_from_IMediaSeeking( IMediaSeeking *iface )
     return CONTAINING_RECORD(iface, AVISplitterImpl, Parser.sourceSeeking.IMediaSeeking_iface);
 }
 
+static inline AVISplitterImpl *impl_from_IBaseFilter(IBaseFilter *iface)
+{
+    return CONTAINING_RECORD(iface, AVISplitterImpl, Parser.filter.IBaseFilter_iface);
+}
+
 /* The threading stuff cries for an explanation
  *
  * PullPin starts processing and calls AVISplitter_first_request
@@ -794,7 +799,7 @@ static HRESULT AVISplitter_ProcessStreamList(AVISplitterImpl * This, const BYTE
         case ckidSTREAMNAME:
             TRACE("processing stream name\n");
             /* FIXME: this doesn't exactly match native version (we omit the "##)" prefix), but hey... */
-            MultiByteToWideChar(CP_ACP, 0, (LPCSTR)(pChunk + 1), pChunk->cb, piOutput.achName, sizeof(piOutput.achName) / sizeof(piOutput.achName[0]));
+            MultiByteToWideChar(CP_ACP, 0, (LPCSTR)(pChunk + 1), pChunk->cb, piOutput.achName, ARRAY_SIZE(piOutput.achName));
             break;
         case ckidSTREAMHANDLERDATA:
             FIXME("process stream handler data\n");
@@ -955,10 +960,8 @@ static HRESULT AVISplitter_InitializeStreams(AVISplitterImpl *This)
 
         for (x = 0; x < This->Parser.cStreams; ++x)
         {
-            if ((DWORD)This->streams[x].frames != This->streams[x].streamheader.dwLength)
-            {
-                FIXME("stream %u: frames found: %u, frames meant to be found: %u\n", x, (DWORD)This->streams[x].frames, This->streams[x].streamheader.dwLength);
-            }
+            if (This->streams[x].frames != This->streams[x].streamheader.dwLength)
+                FIXME("stream %u: frames found: %u, frames meant to be found: %u\n", x, This->streams[x].frames, This->streams[x].streamheader.dwLength);
         }
 
     }
@@ -1027,6 +1030,7 @@ static HRESULT AVISplitter_Disconnect(LPVOID iface);
 static HRESULT AVISplitter_InputPin_PreConnect(IPin * iface, IPin * pConnectPin, ALLOCATOR_PROPERTIES *props)
 {
     PullPin *This = impl_PullPin_from_IPin(iface);
+    AVISplitterImpl *pAviSplit = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
     HRESULT hr;
     RIFFLIST list;
     LONGLONG pos = 0; /* in bytes */
@@ -1036,8 +1040,6 @@ static HRESULT AVISplitter_InputPin_PreConnect(IPin * iface, IPin * pConnectPin,
     ULONG x;
     DWORD indexes;
 
-    AVISplitterImpl * pAviSplit = (AVISplitterImpl *)This->pin.pinInfo.pFilter;
-
     hr = IAsyncReader_SyncRead(This->pReader, pos, sizeof(list), (BYTE *)&list);
     pos += sizeof(list);
 
@@ -1266,7 +1268,7 @@ static HRESULT AVISplitter_Disconnect(LPVOID iface)
 
 static ULONG WINAPI AVISplitter_Release(IBaseFilter *iface)
 {
-    AVISplitterImpl *This = (AVISplitterImpl *)iface;
+    AVISplitterImpl *This = impl_from_IBaseFilter(iface);
     ULONG ref;
 
     ref = InterlockedDecrement(&This->Parser.filter.refCount);
@@ -1421,7 +1423,7 @@ static const IBaseFilterVtbl AVISplitterImpl_Vtbl =
     Parser_SetSyncSource,
     Parser_GetSyncSource,
     Parser_EnumPins,
-    Parser_FindPin,
+    BaseFilterImpl_FindPin,
     Parser_QueryFilterInfo,
     Parser_JoinFilterGraph,
     Parser_QueryVendorInfo
@@ -1429,6 +1431,7 @@ static const IBaseFilterVtbl AVISplitterImpl_Vtbl =
 
 HRESULT AVISplitter_create(IUnknown * pUnkOuter, LPVOID * ppv)
 {
+    static const WCHAR sink_name[] = {'i','n','p','u','t',' ','p','i','n',0};
     HRESULT hr;
     AVISplitterImpl * This;
 
@@ -1445,7 +1448,11 @@ HRESULT AVISplitter_create(IUnknown * pUnkOuter, LPVOID * ppv)
     This->streams = NULL;
     This->oldindex = NULL;
 
-    hr = Parser_Create(&(This->Parser), &AVISplitterImpl_Vtbl, &CLSID_AviSplitter, AVISplitter_Sample, AVISplitter_QueryAccept, AVISplitter_InputPin_PreConnect, AVISplitter_Flush, AVISplitter_Disconnect, AVISplitter_first_request, AVISplitter_done_process, NULL, AVISplitter_seek, NULL);
+    hr = Parser_Create(&This->Parser, &AVISplitterImpl_Vtbl, &CLSID_AviSplitter,
+            sink_name, AVISplitter_Sample, AVISplitter_QueryAccept,
+            AVISplitter_InputPin_PreConnect, AVISplitter_Flush,
+            AVISplitter_Disconnect, AVISplitter_first_request,
+            AVISplitter_done_process, NULL, AVISplitter_seek, NULL);
 
     if (FAILED(hr))
         return hr;
