@@ -76,7 +76,6 @@ static UINT WM_MSIME_RECONVERT;
 static UINT WM_MSIME_QUERYPOSITION;
 static UINT WM_MSIME_DOCUMENTFEED;
 
-
 static HIMC RealIMC(HIMC hIMC)
 {
     if (hIMC == FROM_MACDRV)
@@ -144,8 +143,8 @@ static int updateField(DWORD origLen, DWORD origOffset, DWORD currentOffset,
 
 static HIMCC updateCompStr(HIMCC old, LPCWSTR compstr, DWORD len, DWORD *flags)
 {
-    /* we need to make sure the CompStr, CompClaus and CompAttr fields are all
-     * set and correct */
+    /* We need to make sure the CompStr, CompClause and CompAttr fields are all
+     * set and correct. */
     int needed_size;
     HIMCC   rc;
     LPBYTE newdata = NULL;
@@ -840,6 +839,7 @@ BOOL WINAPI NotifyIME(HIMC hIMC, DWORD dwAction, DWORD dwIndex, DWORD dwValue)
 
                     TRACE("NI_COMPOSITIONSTR: CPS_CANCEL\n");
 
+                    macdrv_clear_ime_text();
                     if (lpIMC->hCompStr)
                         ImmDestroyIMCC(lpIMC->hCompStr);
 
