@@ -548,6 +548,8 @@ void read_undef_symbols( DLLSPEC *spec, char **argv )
     name = ldcombine_files( spec, argv );
 
     cmd = strmake( "%s -u %s", prog, name );
+    if (verbose)
+        fprintf( stderr, "%s\n", cmd );
     if (!(f = popen( cmd, "r" )))
         fatal_error( "Cannot execute '%s'\n", cmd );
 
@@ -1248,13 +1250,13 @@ void output_stubs( DLLSPEC *spec )
             output( "\tcall %s\n", asm_name("__wine_spec_unimplemented_stub") );
             break;
         case CPU_x86_64:
-            output( "\tsubq $8,%%rsp\n" );
+            output( "\tsubq $0x28,%%rsp\n" );
             output_cfi( ".cfi_adjust_cfa_offset 8" );
-            output( "\tleaq .L__wine_spec_file_name(%%rip),%%rdi\n" );
+            output( "\tleaq .L__wine_spec_file_name(%%rip),%%rcx\n" );
             if (exp_name)
-                output( "leaq .L%s_string(%%rip),%%rsi\n", name );
+                output( "leaq .L%s_string(%%rip),%%rdx\n", name );
             else
-                output( "\tmovq $%d,%%rsi\n", odp->ordinal );
+                output( "\tmovq $%d,%%rdx\n", odp->ordinal );
             output( "\tcall %s\n", asm_name("__wine_spec_unimplemented_stub") );
             break;
         case CPU_ARM:
@@ -1384,7 +1386,8 @@ static void build_windows_import_lib( DLLSPEC *spec )
             m_flag = NULL;
             break;
     }
-    strarray_add( &args, "-k", "-l", output_file_name, "-d", def_file, NULL );
+    strarray_add( &args, "-k", strendswith( output_file_name, ".delay.a" ) ? "-y" : "-l",
+                  output_file_name, "-d", def_file, NULL );
     if (m_flag)
         strarray_add( &args, "-m", m_flag, as_flags, NULL );
     spawn( args );
